@@ -1,5 +1,5 @@
 "factor2cluster" <-
-function (loads,cut=.0) 
+function (loads,cut=.0,aslist=FALSE) 
 {
     
      if (!is.matrix(loads) ) {l <-loads$loadings} else {l <- loads}
@@ -17,6 +17,7 @@ function (loads,cut=.0)
   colnames(factor2cluster) <- colnames(l)
    nitems <- colSums(abs(factor2cluster))
    for (i in ncols:1) {if (nitems[i]<1) {factor2cluster <- factor2cluster[,-i,drop=FALSE]} }#remove columns with no variables
+   if(aslist) factor2cluster <- keys2list(factor2cluster)
     return(factor2cluster)
 }
 
