@@ -23,7 +23,8 @@ function(R,digits=2,minlength=5) {
 	
 	if(k1 * nvar < width) {k1 <- nvar}  
 	k1 <- floor(k1)
-	fx <- format(round(R,digits=digits))
+	if(!is.character(R) ) {
+	fx <- format(round(R,digits=digits))} else {fx <- format(R)}
 	 if(nrow(R) == ncol(R) ) {fx[!lowleft] <- ""}
 	 for(k in seq(0,nvar,k1)) { if(k<nvar) {
 	print(fx[(k+1):nvar,(k+1):min((k1+k),nvar)],quote=FALSE)}
@@ -32,6 +33,8 @@ function(R,digits=2,minlength=5) {
 	
 "lowerCor" <- 
 function(x,digits=2,use="pairwise",method="pearson",minlength=5) {
+nvar <- NCOL(x)
+x <- char2numeric(x)    #added 1/2/21
    R <- cor(x,use=use,method=method)
    lowerMat(R,digits,minlength=minlength)
    invisible(R)
@@ -441,7 +444,7 @@ function (f,m, dictionary,cut=.3, digits = 2) {
         ord <- rownames(f)
         nfact <- ncol(f)
     }
-    means <- m[ord]
+    means <- m[ord]   #incorrectly added a comma to allow it sort dataframes 6/22/21
     f <- data.frame(unclass(f),means=means)
 
     contents <- lookup(rownames(f), y=dictionary)
@@ -577,15 +580,18 @@ text(x,y-2,"Unreliable but Valid")
 # print(round(R,digits))
 # invisible(R)}
 
-#should we add in the abilty to not choke on character variables?
+#finally added the char2numeric so it will not choke on character variables   1/3/21
 
 "cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson") {
 multi <- FALSE
 if(is.list(x) && is.null(y)) {multi <- TRUE
  n <- length(x)
 xi <- x[[1]]
- for (i in 2:n) {xi <- cbind(xi,x[[i]])} 
+ for (i in 2:n) {xi <- cbind(xi,x[[i]])}
+  
 R <- cor(xi,use=use,method=method) }else {
+x <- char2numeric(x)
+y <- char2numeric(y)
 R <- cor(x,y,use=use,method=method)}
 if(multi) {lowerMat(R,digits) } else {print(round(R,digits))}
 invisible(R)}
@@ -612,30 +618,41 @@ pretty}
 #October 25, 2016
 #June 18 2020  added  as.factor to convert character strings that are not stored as factors
 #this has a downsize that it converts numbers stored as characters to factors (see nchar2numeric)
-"char2numeric" <- function(x) {
+#added the flag option and the change the colname option 1/2/21
+
+"char2numeric" <- function(x,flag=TRUE) {
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
                                   if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) 
-                          } else {x[[i]] <- NA} }
+                          } else {x[[i]] <- NA}
+                         if(flag) colnames(x)[i] <- paste0(colnames(x)[i],"*")
+                          } 
+                          
               } 
               invisible(x)} 
-#added June 25, 2020 to handle the case of numeric data stored as characters              
-"nchar2numeric" <- function(x) {
+              
+              
+#added June 25, 2020 to handle the case of numeric data stored as characters  
+#added flag and colnames option  1/2/21             
+"nchar2numeric" <- function(x,flag=TRUE) {
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  { 
                 if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {
                                   if(is.factor(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) } else {x[[i]] <- as.numeric(x[[i]])}
-                          } else {x[[i]] <- NA} }
+                          } else {x[[i]] <- NA}
+                          if(flag) colnames(x)[i] <- paste0(colnames(x)[i],"*")
+                            }
               } 
               invisible(x)} 
 
 
 #this just shows if it is a matrix is symmetric and has diagonals of 1
+#Added the unclass to  handle a problem with class partial.r  4/10/21
 "isCorrelation" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
-  if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
+  if( is.data.frame(x)) {if(isSymmetric(unclass(unname(as.matrix(x))))) { value <- TRUE}} else {if(isSymmetric(unclass(unname(x)))) {value <- TRUE}}}
   value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )
   value <- value && isTRUE((min(x,na.rm=TRUE)>= -1) & (max(x,na.rm=TRUE) <= 1))  
   return(value)}
@@ -643,7 +660,7 @@ pretty}
   #this just shows if it is a symmetric matrix
 "isCovariance" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
-  if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
+  if( is.data.frame(x)) {if(isSymmetric(unclass(unname(as.matrix(x))))) { value <- TRUE}} else {if(isSymmetric(unclass(unname(x)))) {value <- TRUE}}}
  # value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )  #don't check for diagonal of 1
   return(value)}
   
@@ -671,6 +688,31 @@ flip <- function(R,key) {#reverse correlations with keys  < 0
 	R[,key<0] <- -R[,key < 0]
   return(R)}
     
-    
- 
+
+ matMult <- function(A,B) {  #matrix multiplication without matrices!  
+  nvar1 <- ncol(A)
+  nvar2 <- ncol(B)
+  M <- matrix(NA,ncol=nvar1,nrow=nrow(B))
+    if(nrow(A) != nvar2) {stop("incompatible dimensions")}
+   for(i in 1:nvar1) {
+    for (j in 1:nrow(B))  {
+    M[j,i] <-  sum(A[,i] * B[j,], na.rm=TRUE)
+    } 
+    }
+    return(M)}
+     
+   
+   
+   factorScoresSapa  <- function(weights,items) {
+  nf <- NCOL(weights)
+  scores<- matrix(NA,nrow=NROW(items),ncol=nf)
+  n.obs <- NROW(items)
+  n.items <- NCOL(items)
+  titem <- t(items)
+  for (factors in (1:nf)) {
+           
+    scores[,factors] <-t(colMeans( weights[,factors] * titem,  na.rm=TRUE))
+    }
+    return(scores)
+  }
 
